use std::sync::{Arc, LazyLock};

use norun_grpc_interface::Project;
use tokio::sync::Mutex;

use crate::state::ServerState;

#[derive(Clone)]
pub struct RegistryService {
    store: Arc<Mutex<Vec<Project>>>,
}

impl RegistryService {
    pub async fn store(&self, project: &Project) -> anyhow::Result<()> {
        tracing::debug!("storing project");

        let mut store = self.store.lock().await;

        store.push(project.clone());

        tracing::debug!("currently has {} in storage", store.len());

        Ok(())
    }

    pub async fn get_topic(&self, topic: &str) -> anyhow::Result<Projects> {
        tracing::debug!("get projects for topic");

        let projects = {
            let store = self.store.lock().await;
            store
                .iter()
                .filter(|i| i.name == topic)
                .cloned()
                .collect::<Vec<_>>()
        };

        Ok(Projects { projects })
    }
}

pub struct Projects {
    pub projects: Vec<Project>,
}

pub trait RegistryServiceState {
    fn registry_service(&self) -> RegistryService;
}

impl RegistryServiceState for ServerState {
    fn registry_service(&self) -> RegistryService {
        static SERVICE: LazyLock<RegistryService> = LazyLock::new(|| RegistryService {
            store: Arc::default(),
        });

        SERVICE.clone()
    }
}
